# >>> ../addons/eventscripts/votemap/votemap.py
# >>> release 1.0 by Steven Hartin
# To edit see ../votemap/config.cfg
# Do not change this file, things could break if you did.

import es
import popuplib
import playerlib
import votelib
import time
import gamethread
import services
import os
import langlib

import psyco
psyco.full()

info = es.AddonInfo()
info.name = "VoteMap"
info.version = "1.0"
info.author = "Freddukes"
info.url = "http://forums.mattie.info/cs/"
info.description = "Allows you to group maps together in a simple vote"

dict_options = {
'rounds_before_mp_maxrounds':  [2,   'How many rounds before mp_maxrounds the VoteMap menu appears'],
'rounds_before_mp_winlimit':   [2,   'How many rounds before mp_winlimit the VoteMap menu appears'],
'time_before_end_of_map_vote': [180, 'How many seconds befre mp_timelimit the VoteMap menu appears'],
'rounds_before_gg_win':        [2,   'How many GG levels before the win level the VoteMap menu appears'],
'announce_votes':              [1,   'Whether or not the votes will be announce || 1 = enable, 0 = Disable'],
'change_map':                  [1,   'The way in which the map will Change after a vote. 1 = end of map, 2 = end of round 3 = now'],
'allow_rtv':                   [1,   'Whether or not RTV is allowed'],
'rtv_percentage':              [60,  'The amount of percentage of people that must type RTV for it to become active'],
'min_players_needed':          [1,   'The minimum players on the server before RTV can be active'],
'rtv_delay_timer':             [90,  'The time in seconds after a new map that you can RTV'],
'rtv_change_map':              [1,   'The way in which RTV changes level. 1 = End of round, 2 = Instantly'],
'easy_install':                [1,   'Search through ../maps/ adds all the maps and sorts them']
}

es.ServerVar("votemap_version",info.version,"VoteMap Version: The current version of VoteMap").makepublic()

maplist = {}
rtvdict = {}
PlayerDict = {}
mapdict = {}
round_count = {}
single_map = None
text = None
vm_gg_vote_start = 20

def load():
    global text
    
    es.msg("#multi","#lightgreen[VoteMap] #greenInitialising VoteMap")
    
    es.dbgmsg(0,'\n*******************************************************')
    es.dbgmsg(0,'VoteMap: The new way to vote for maps')
    es.dbgmsg(0,'VoteMap: Current Version - %s'%info.version)
    es.dbgmsg(0,'VoteMap: Made by Freddukes')
    
    for option in dict_options:
        es.ServerVar(option, dict_options[option][0], dict_options[option][1])
    
    setup_auth()
    es.dbgmsg(0,'VoteMap: Successfully Set up Authorization Services.')
    
    if not es.exists('command','addmap'):
        es.regcmd('addmap','votemap/add_map')
    if not es.exists('saycommand','changelevel'):
        es.regsaycmd('changelevel','votemap/changelevel')
    if not es.exists('saycommand','startvote'):
        es.regsaycmd('startvote','votemap/startvote')
    
    langpath = es.getAddonPath('votemap') + '/strings.ini'
    if os.path.isfile(langpath):
        text = langlib.Strings(langpath)
        es.dbgmsg(0,'VoteMap: Successfully Loaded \'strings.ini\'')
    else:
        es.dbgmsg(0,'VoteMap: - ERROR Unable to load languages.ini! Please ensure it is in the ./votemap/ directory.')
    if not text:
        text = 'Missing Language'
    
    configpath = es.getAddonPath('votemap') + '/config.cfg'
    if os.path.isfile(configpath):
        es.server.cmd('es_xmexec ../addons/eventscripts/votemap/config.cfg')
        es.dbgmsg(0,'VoteMap: Successfully Loaded \'config.cfg\'')
    else:
        es.dbgmsg(0,'VoteMap: - ERROR Unable to load config.cfg! Please ensure it is in the ./votemap/ directory.')
        
    mappath = es.getAddonPath('votemap') + '/maps.cfg'
    if os.path.isfile(mappath):
        es.server.cmd('es_xmexec ../addons/eventscripts/votemap/maps.cfg')
        es.dbgmsg(0,'VoteMap: Successfully Loaded \'maps.cfg\'')
    else:
        es.dbgmsg(0,'VoteMap: - ERROR Unable to load maps.cfg! Please ensure it is in the ./votemap/ directory.')
    
    if int(es.ServerVar('easy_install')):
        for (path, dirs, files) in os.walk(str(es.ServerVar('eventscripts_gamedir')).replace('\\','/') + '/maps'):
            for mapfile in files:
    		  if str(mapfile).endswith('.bsp'):
    		      Maps(mapfile.replace('.bsp',''))
    
    map_dict = Maps().return_map_list()
    map_list = sorted(map_dict)

    mapmainmenu = popuplib.easymenu('mapmainmenu','_popup_choice', register_vote)
    nominatemenu = popuplib.easymenu('nominatemenu','_popup_choice', nominated_a_vote)
    
    mapmainmenu.settitle("[Map Vote]")
    nominatemenu.settitle("[Nominate]")
    for temp_map in map_list:
        if len(map_dict[temp_map]):
            mapmainmenu.addoption(temp_map, temp_map)
            nominatemenu.addoption(temp_map, temp_map)
    es.dbgmsg(0,'VoteMap: Finished Loading!')
    es.dbgmsg(0,'*******************************************************\n')
           
def gg_levelup(ev):
    if int(es.ServerVar('gg_leader_lvl')) == vm_gg_vote_start:
        menusend()
        
def player_activate(ev):
    userid = int(ev['userid'])
    PlayerDict[int(ev['userid'])] = {'nominated':0,'rtv':0}
    
def player_disconnect(ev):
    del PlayerDict[int(ev['userid'])]
    
def es_map_start(ev):
    global current_max_value
    global currentindex
    global end_now
    global rtv_active
    global rtv_began
    global mapstart
    global rtv_timer_passed
    global vote_started
    global round_count
    global rtv_votes
    global nominatedatleastonemap
    for vote_name in ['vote_map', 'single_vote']:
        if votelib.exists(vote_name):
            if votelib.isrunning(vote_name):
                vote = votelib.find(vote_name)
                vote.stop(True)
            votelib.delete(vote_name)
    rtvdict.clear()
    gamethread.cancelDelayed('thirty_second_vote')
    round_count = {'round_counting':0,'t_wins':0,'c_wins':0,'frag_kills':0}
    es.ServerVar('vm_gg_vote_start',0)
    vote_started, rtv_timer_passed, rtv_active, rtv_began, end_now, current_max_value, current_index, rtv_votes, nominatedatleastonemap = 0, 0, 0, 0, 0, 0, 0, 0, 0
    mapstart = time.time()
    timelimit = es.ServerVar('mp_timelimit')
    if int(es.ServerVar('rtv_delay_timer')):
        gamethread.cancelDelayed('rtv_delay_timer')
        gamethread.delayedname(int(es.ServerVar('rtv_delay_timer')), 'time_expired', set_rtv_timer_passed)
    else:
        rtv_timer_passed = 1
    if int(es.ServerVar('time_before_end_of_map_vote')):
        gamethread.delayed(10, DelayTimer)
    if int(es.ServerVar('rounds_before_gg_win')):
        gamethread.delayed(10, DelayGunGameSetting)
    
def DelayGunGameSetting():
    global vm_gg_vote_start
    if es.exists("variable","gg_getmaxlevel"):
        es.ServerVar('vm_gg_vote_start',0)
        es.server.queuecmd('gg_getmaxlevel vm_gg_vote_start')
        vm_gg_vote_start = int(es.ServerVar('vm_gg_vote_start'))
        vm_gg_vote_start -= int(es.ServerVar('rounds_before_gg_win'))
    
def DelayTimer():
    gamethread.cancelDelayed('votemap_timer')
    delay = int(es.ServerVar('mp_timelimit')) * 60 - int(es.ServerVar('time_before_end_of_map_vote')) - 10
    gamethread.delayedname(delay, 'votemap_timer', menusend)
    
def set_rtv_timer_passed():
    global rtv_timer_passed
    rtv_timer_passed = 1
    
def server_cvar(ev):
    if ev['cvarname'] == "mp_timelimit" and mapstart <> None and int(es.ServerVar('time_before_end_of_map_vote')):
        gamethread.cancelDelayed('votemap_timer')
        delay = es.ServerVar('mp_timelimit') * 60 - int(es.ServerVar('time_before_end_of_map_vote')) - (time.time() - mapstart)
        if delay:
            gamethread.delayedname(delay, 'votemap_timer', menusend)
        else:
            menusend()      
    
def round_end(ev):
    round_count['round_counting'] += 1
    winlimit = int(es.ServerVar('mp_winlimit')) - int(es.ServerVar('rounds_before_mp_winlimit'))
    roundlimit = int(es.ServerVar('mp_maxrounds')) - int(es.ServerVar('rounds_before_mp_maxrounds'))
    if ev['winner'] == '2':
        round_count['t_wins'] += 1
    elif ev['winner'] == '3':
        round_count['c_wins'] += 1
    if (round_count['round_counting'] >= roundlimit) and roundlimit > 0:
        menusend()
    elif (round_count['c_wins'] >= winlimit or round_count['t_wins'] >= winlimit) and winlimit > 0:
        menusend()
    if not rtv_active:
        if end_now:
            if int(es.ServerVar('change_map')) == 2:
                nextmap = str(es.ServerVar('eventscripts_nextmapoverride'))
                if nextmap not in ["None", "0"]:
                    tokens = {}
                    tokens['nextmap'] = nextmap
                    for player in playerlib.getPlayerList('#human'):
                        es.tell(int(player), "#multi", text("changing level", tokens, player.get("lang")))
                    gamethread.delayed(3,end_round)
    elif int(es.ServerVar('rtv_change_map')) == 1:
        if end_now:
            nextmap = str(es.ServerVar('eventscripts_nextmapoverride'))
            if nextmap not in ["None","0"]:
                tokens = {}
                tokens['nextmap'] = nextmap
                for player in playerlib.getPlayerList('#human'):
                    es.tell(int(player), "#multi", text("changing level", tokens, player.get("lang")))
                gamethread.delayed(3,end_round)
                
def player_say(ev):
    global rtv_active
    global rtv_started
    playertext = ev['text'].lower()
    userid = int(ev['userid'])
    if playertext == "rtv":
        if int(es.ServerVar('allow_rtv')):
            if rtv_timer_passed:
                rtvoted = PlayerDict[userid]['rtv']
                if not rtvoted:
                    playercount = len(filter(lambda x: not es.getplayersteamid(x) == "BOT", es.getUseridList()))
                    if playercount >= int(es.ServerVar('min_players_needed')):
                        if not rtv_active:
                            check_rtv(userid)
                        else:
                            tokens = {}
                            tokens['state'] = 'RTV'
                            es.tell(userid,'#green',text("rtv active", tokens, playerlib.getPlayer(userid).get("lang")))
                    else:
                        tokens = {}
                        tokens['players'] = str(es.ServerVar('min_players_needed'))
                        tokens['state']   = 'RTV'
                        es.tell(userid, '#green', text("rtv players", tokens, playerlib.getPlayer(userid).get("lang")))
                else:
                    tokens = {}
                    tokens['state'] = "RTV"
                    es.tell(userid, '#green', text("rtv already", tokens, playerlib.getPlayer(userid).get("lang"))) 
            else:
                tokens = {}
                tokens['state'] = 'RTV'
                tokens['seconds']  = str(es.ServerVar('rtv_delay_timer'))
                es.tell(userid, '#green', text("rtv timer not passed", tokens, playerlib.getPlayer(userid).get("lang")))
        else: 
            tokens = {}
            tokens['state'] = 'RTV'
            es.tell(userid, '#green', text("rtv not allowed", tokens, playerlib.getPlayer(userid).get("lang")))
    elif playertext == "nominate":
        if int(es.ServerVar('allow_rtv')):
            if rtv_timer_passed == 1:
                nominations = PlayerDict[userid]['nominated']
                if not nominations:
                    playercount = len(filter(lambda x: not es.getplayersteamid(x) == "BOT", es.getUseridList()))
                    if playercount >= int(es.ServerVar('min_players_needed')):
                        if not rtv_active:
                            popuplib.send('nominatemenu',userid)
                        else:
                            tokens = {}
                            tokens['state'] = 'RTV'
                            es.tell(userid,'#green',text("rtv active", tokens, playerlib.getPlayer(userid).get("lang")))
                    else:
                        tokens = {}
                        tokens['players'] = str(es.ServerVar('min_players_needed'))
                        tokens['state']   = 'Nominate'
                        es.tell(userid, '#green', text("rtv players", tokens, playerlib.getPlayer(userid).get("lang")))
                else:
                    tokens = {}
                    tokens['state'] = "Nominate"
                    es.tell(userid, '#green', text("rtv already", tokens, playerlib.getPlayer(userid).get("lang"))) 
            else: 
                tokens = {}
                tokens['state'] = 'Nominate'
                tokens['seconds']  = str(es.ServerVar('rtv_delay_timer'))
                es.tell(userid, '#green', text("rtv timer not passed", tokens, playerlib.getPlayer(userid).get("lang")))
        else:
            tokens = {}
            tokens['state'] = 'RTV'
            es.tell(userid, '#green', text("rtv not allowed", tokens, playerlib.getPlayer(userid).get("lang")))
    elif playertext == "nextmap":
        nextmap = str(es.ServerVar('eventscripts_nextmapoverride'))
        tokens = {}
        if nextmap and nextmap != '0':
            tokens['nextmap'] = "Decided By Vote"
        else:
            tokens['nextmap'] = nextmap
        for player in playerlib.getPlayerList('#human'):
            es.tell(int(player), '#multi', text("nextmap", tokens, player.get("lang")))
    elif playertext == "thetime":
        thetime = time.strftime("%B %d, %Y - %H:%M:%S (%z)")
        tokens = {}
        tokens['time'] = thetime
        for player in playerlib.getPlayerList('#human'):
            es.tell(int(player), '#multi', text("the time", tokens, player.get("lang")))
    elif playertext == "timeleft":
        es.sexec(userid,"timeleft")
        
def check_rtv(userid):
    global rtv_began
    global rtv_votes
    global rtv_active
    global nominatedatleastonemap
    playercount = len(filter(lambda x: not es.getplayersteamid(x) == "BOT", es.getUseridList()))
    username = es.getplayername(userid)
    if not rtv_began:
        rtv_began = 1
        votes_needed = int(round(float(playercount) * int(es.ServerVar('rtv_percentage')) / 100))
        rtv_votes += 1
        tokens = {}
        tokens['name'] = username
        tokens['votes'] = str(votes_needed - 1)
        for player in playerlib.getPlayerList('#human'):
            lang = player.get("lang")
            es.tell(int(player), '#lightgreen', text("rtv started one", tokens, lang))
            es.tell(int(player), '#lightgreen', text("rtv started two", lang=lang))
        PlayerDict[userid]['rtv'] = 1
        if rtv_votes >= votes_needed:
            rtv_active = 1
            if nominatedatleastonemap:
                send_rtv()
            else:
                menusend()
    else:
        rtv_votes += 1
        votes_needed = int(round(float(playercount) * int(es.ServerVar('rtv_percentage')) / 100))
        PlayerDict[userid]['rtv'] = 1
        if rtv_votes >= votes_needed:
            rtv_active = 1
            if nominatedatleastonemap:
                send_rtv()
            else:
                menusend()
        elif int(es.ServerVar('announce_votes')):
            tokens = {}
            tokens['name']  = username
            tokens['votes'] = str(votes_needed - rtv_votes)
            for player in playerlib.getPlayerList('#human'):
                es.tell(int(player), '#lightgreen', text("rtv vote", tokens, player.get("lang")))
            
def send_rtv():
    global vote_started
    rtvmenu = popuplib.easymenu('rtvmenu','_popup_choice', register_vote)
    rtvmenu.settitle('[Rock The Vote]')
    for rtv_map in rtvdict:
        rtvmenu.addoption(rtv_map, rtv_map)
    gamethread.cancelDelayed('time_expired')
    if not vote_started:
        vote_started = 1
        startedzepopup()
        for player in PlayerDict:
            popuplib.send('rtvmenu', player)
        
def register_vote(userid, choice, popupid):
    name = es.getplayername(userid)
    if choice:
        if choice not in mapdict:
            mapdict[choice] = 1
        else:
            mapdict[choice] += 1
    if int(es.ServerVar('announce_votes')):
        tokens = {}
        tokens['name'] = name
        tokens['option'] = choice
        for player in playerlib.getPlayerList('#human'):    
            es.tell(int(player), '#multi', text("vote option", tokens, player.get("lang")))
        
def nominated_a_vote(userid, choice, popupid):
    global nominatedatleastonemap
    global rtvdict
    name = es.getplayername(userid)
    nominatedatleastonemap = 1
    name = es.getplayername(userid)
    if choice not in rtvdict:
        rtvdict[choice] = Maps().return_map_list()[choice]
        if int(es.ServerVar('announce_votes')):
            tokens = {}
            tokens['name']   = name
            tokens['option'] = choice
            for player in playerlib.getPlayerList('#human'):
                es.tell(int(player), '#multi', text('nominate option', tokens, player.get("lang")))
        PlayerDict[userid]['nominated'] = 1
    else:
        tokens = {}
        tokens['option'] = choice
        es.tell(userid,'#green', text("already nominated", tokens, playerlib.getPlayer(userid).get("lang")))
        
def menusend():
    global vote_started
    gamethread.cancelDelayed('time_expired')
    if not vote_started and not rtv_active:
        vote_started = 1
        startedzepopup()
        for x in PlayerDict:
            popuplib.send('mapmainmenu',x)
            
def startedzepopup():
    es.cexec_all('playgamesound','ambient/machines/teleport4.wav')
    for player in playerlib.getPlayerList('#human'):
        es.tell(int(player), '#multi', text('vote started', lang=player.get("lang")))
    gamethread.delayedname(20,'thirty_second_vote',es.centermsg,'You have 10 seconds left to vote')
    gamethread.delayedname(25,'thirty_second_vote',es.centermsg,'You have 5 seconds left to vote')
    gamethread.delayedname(26,'thirty_second_vote',es.centermsg,'4....')
    gamethread.delayedname(27,'thirty_second_vote',es.centermsg,'3...')
    gamethread.delayedname(28,'thirty_second_vote',es.centermsg,'2..')
    gamethread.delayedname(29,'thirty_second_vote',es.centermsg,'1.')
    gamethread.delayedname(30,'thirty_second_vote',es.centermsg,'Voting has ended!')
    gamethread.delayedname(30,'thirty_second_vote',es.msg,('#green','Voting has ended!'))
    gamethread.delayedname(30,'thirty_second_vote',es.cexec_all,'playgamesound ambient/alarms/warningbell1.wav')
    gamethread.delayedname(30,'thirty_second_vote',voteblock)
    
def voteblock():
    map_style_winner = "Hostage Rescue (cs)"
    a = sorted(mapdict, MySort)
    try:
        map_style_winner = str(a[0])
    except IndexError:
        return
    vote_question = "What should be the next %s map?"%map_style_winner
    votetemp = votelib.create('vote_map',vote_win,vote_msg)
    votetemp.setquestion(vote_question)
    temp_key = Maps().return_map_list()[map_style_winner]
    for mapname in temp_key:
        votetemp.addoption(mapname)
    gamethread.delayedname(5,'thirty_second_vote',es.cexec_all,'playgamesound ambient/machines/teleport4.wav')
    gamethread.delayedname(5,'thirty_second_vote',votetemp.start,'30')
    
def MySort(map1, map2):
    if mapdict[map1] > mapdict[map2]: return -1
    if mapdict[map1] == mapdict[map2]: return 0
    return 1
    
def vote_msg(userid, votename, optionid, option):
    if int(es.ServerVar('announce_votes')):
        if int(optionid):
            tokens = {}
            tokens['name']   = es.getplayername(userid)
            tokens['option'] = option
            for player in playerlib.getPlayerList('#human'):
                es.tell(int(player), '#multi', text("vote option", tokens, player.get("lang")))
        
def vote_win(popupid, optionid, optiontext, optionvotes, optionpercent, totalvotes, wastied, wascanceled):
    global end_now
    es.cexec_all('playgamesound','ambient/alarms/warningbell1.wav')
    es.set('eventscripts_nextmapoverride',optiontext)
    nextmap = es.ServerVar('eventscripts_nextmapoverride')
    tokens = {}
    tokens['nextmap'] = nextmap
    for player in playerlib.getPlayerList('#human'):
        es.tell(int(player), '#multi', text("nextmap", tokens, player.get("lang")))
    if not rtv_active:
        if int(es.ServerVar('change_map')) == 2:
            end_now = 1
        elif int(es.ServerVar('change_map')) == 3:
            if nextmap:
                for player in playerlib.getPlayerList('#human'):
                    es.tell(int(player), "#multi", text("changing level", tokens, player.get("lang")))
                gamethread.delayed(3,end_round)
    elif int(es.ServerVar('rtv_change_map')) == 1:
        end_now = 1
    elif int(es.ServerVar('rtv_change_map')) == 2:
        if nextmap:
            for player in playerlib.getPlayerList('#human'):
                es.tell(int(player), "#multi", text("changing level", tokens, player.get("lang")))
            gamethread.delayed(3,end_round)

def end_round():
    userid = es.getuserid()
    if not userid:
        return
    es.server.queuecmd('nextlevel ' + es.ServerVar('eventscripts_nextmapoverride'))
    es.server.queuecmd('es_give %s game_end'%userid)
    es.server.queuecmd('es_fire %s game_end EndGame'%userid)

def add_map():
    mapname = str(es.getargv(1))
    maptype = es.getargv(2)
    if not maptype:
        Maps(mapname)   
    else:
        maptype = str(maptype)
        Maps(mapname, maptype)
            
class Maps:
    map_list = {}
    all_maps = []
    
    def __init__(self, mapname="", maptype=""):
        if mapname:
            self.mapname = mapname
            if not maptype:
                self.add_default_map(mapname)
            else:
                self.add_custom_map(mapname, maptype)
            
    def add_default_map(self, mapname):
        dict_string = ''
        mapname = mapname.lower()
        a = mapname.startswith
        if a('cs_'):
            dict_string = "Hostage Rescue (cs)"
        elif a('de_'):
            dict_string = "Detonation (de)"
        elif a('fy_'):
            dict_string = "Fight Yard (fy)"
        elif a(('zm_','zh','zombie')):
            dict_string = "Zombie"
        elif a('gg_'):
            dict_string = "GunGame (gg)"
        elif a('surf_'):
            dict_string = "Surf"
        elif a('dm_'):
            dict_string = "Death Match (dm)"
        elif a('aim_'):
            dict_string = "Aim Texture"
        elif a('awp_'):
            dict_string = "AWP Snipers (awp)"
        elif a('glass'):
            dict_string = "Glass"
        elif a('he_'):
            dict_string = "Grenade Maps (he)"
        elif a('ka_'):
            dict_string = "Kife Arena (ka)"
        elif a('scout'):
            dict_string = "Scout Maps"
        else:
            dict_string = "Miscellaneous"
        if not self.map_list.has_key(dict_string):
            self.map_list[dict_string] = []
        if not mapname in self.all_maps:
            self.map_list[dict_string].append(mapname)
            self.all_maps.append(mapname)
        
    def add_custom_map(self, mapname, maptype):
        if not self.map_list.has_key(maptype):
            self.map_list[maptype] = []
        if not mapname in self.all_maps:
            self.map_list[maptype].append(mapname)
            self.all_maps.append(mapname)
        
    def return_map_list(self):
        return self.map_list
        
def setup_auth():
    global isAuthed
    if services.isRegistered('auth'):
        auth_service = services.use('auth')
        auth_service.registerCapability('votemap_admin', auth_service.ADMIN)
        isAuthed = lambda x: auth_service.isUseridAuthorized(x, 'votemap_admin')
    else:
        isAuthed = lambda x: False

def changelevel():
    userid = es.getcmduserid()
    if isAuthed(userid):
        level = str(es.getargv(1))
        if os.path.isfile(str(es.ServerVar('eventscripts_gamedir')).replace('\\','/') + '/maps/%s.bsp'%level):
            es.set('eventscripts_nextmapoverride',level)
            nextlevel = str(es.ServerVar('eventscritps_nextmapoverride'))
            tokens = {}
            tokens['nextmap'] = nextlevel
            for player in playerlib.getPlayerList('#human'):
                es.tell(int(player), "#multi", text("changing level", tokens, player.get("lang")))
            gamethread.delayed(3,end_round)
        else:
            tokens = {}
            tokens['level'] = level
            es.tell(userid,'#lightgreen', text("no map", tokens, playerlib.getPlayer(userid).get("lang")))
            
def startvote(): 
    userid = es.getcmduserid()
    if isAuthed(userid):
        main_popup = popuplib.easymenu('admin_map_menu', '_popup_choice', SubMapGroup)
        main_popup.settitle('[Select a map group]')
        map_dict = Maps().return_map_list()
        map_list = sorted(map_dict)
        for map_group in map_list:
            if len(map_dict[map_group]):
                main_popup.addoption(map_group, map_group)
        popuplib.send('admin_map_menu', userid)
                
def SubMapGroup(userid, choice, popupid):
    map_dict = Maps().return_map_list()
    a = popuplib.easymenu('sub_admin_map_menu', '_popup_choice', SelectChangeMap)
    a.settitle(choice)
    for mapname in map_dict[choice]:
        if mapname:
            a.addoption(mapname, mapname)
    a.submenu(10, 'admin_map_menu')
    popuplib.send('sub_admin_map_menu', userid)
    
def SelectChangeMap(userid, choice, popupid):
    global single_map
    if os.path.isfile(str(es.ServerVar('eventscripts_gamedir')).replace('\\','/') + '/maps/%s.bsp'%choice):
        single_map = choice
        myVote = votelib.create('single_vote', single_vote_win, single_vote_msg)
        myVote.setquestion('Do you want %s to be the next map?'%choice)
        myVote.addoption('Yes')
        myVote.addoption('No')
        myVote.start(30)
        es.cexec_all('playgamesound','ambient/machines/teleport4.wav')
        
def single_vote_msg(userid, votename, optionid, option):
    if int(es.ServerVar('announce_votes')):
        if int(optionid):
            tokens = {}
            tokens['name']   = es.getplayername(userid)
            tokens['option'] = option 
            for player in playerlib.getPlayerList('#human'):
                es.tell(int(player), '#multi', text('vote option', tokens, player.get("lang")))
        
def single_vote_win(popupid, optionid, optiontext, optionvotes, optionpercent, totalvotes, wastied, wascanceled):
    global end_now
    es.cexec_all('playgamesound','ambient/alarms/warningbell1.wav')
    if single_map:
        if optiontext.lower() == "yes":
            es.set('eventscripts_nextmapoverride', single_map)
            nextmap = es.ServerVar('eventscripts_nextmapoverride')
            tokens = {}
            tokens['nextmap'] = nextmap
            for player in playerlib.getPlayerList('#human'):
                es.tell(int(player), "#multi", text("changing level", tokens, player.get("lang")))
            gamethread.delayed(3,end_round)
